@@ -1,8 +1,8 @@
 #!/bin/sh
 
-# $Id: filter.sh,v 3.0.1.4 1999/01/13 18:03:19 ram Exp ram $
+# $Id: filter.sh,v 3.0.1.3 1995/08/07 16:06:52 ram Exp $
 #
-#  Copyright (c) 1990-2006, Raphael Manfredi
+#  Copyright (c) 1990-1993, Raphael Manfredi
 #  
 #  You may redistribute only under the terms of the Artistic License,
 #  as specified in the README file that comes with the distribution.
@@ -11,9 +11,6 @@
 #  of the source tree for mailagent 3.0.
 #
 # $Log: filter.sh,v $
-# Revision 3.0.1.4  1999/01/13  18:03:19  ram
-# patch64: agent.wait is now stored in the spool rather than in queue
-#
 # Revision 3.0.1.3  1995/08/07  16:06:52  ram
 # patch37: avoid forking of a new mailagent if one is sitting in background
 #
@@ -34,9 +31,7 @@
 #    "|exec /users/ram/mail/filter >>/users/ram/.bak 2>&1"
 
 # Variable HOME *must* correctly be set to your home directory
-# This should compute it correctly, if not, hardwire it
-user=`(logname || whoami) 2>/dev/null`
-HOME=`grep ^$user: /etc/passwd | cut -d':' -f 6`
+HOME=/york/ram
 export HOME
 
 # The PATH variable must also correctly be set. This variable will be
@@ -235,6 +230,7 @@ if cmp $temp $qtemp >/dev/null 2>&1; then
 fi
 
 # Change the name of the temporary file.
+user=`(logname || whoami) 2>/dev/null`
 tmpdir=`echo $temp | sed -e 's/\(.*\)\/.*/\1/'`
 mv $temp $tmpdir/$user.$$
 temp="$tmpdir/$user.$$"
@@ -251,9 +247,9 @@ eval $addlog
 # is already updating the agent.wait file. The worse that could result from
 # this shallowness would be having an unprocessed mail.
 umask 077
-set "WARNING mailagent ignores mail was left in $temp"
-if /bin/echo "$temp" >> $spool/agent.wait; then
-	if grep "$temp" $spool/agent.wait >/dev/null 2>&1; then
+set 'WARNING mailagent ignores where mail was left'
+if /bin/echo "$temp" >> $queue/agent.wait; then
+	if grep "$temp" $queue/agent.wait >/dev/null 2>&1; then
 		set "NOTICE $temp memorized into agent.wait"
 	fi
 fi
