@@ -14,7 +14,6 @@ esac
 case "$0" in
 */*) cd `expr X$0 : 'X\(.*\)/'` ;;
 esac
-revision=`awk '/^#define[ 	]*REVISION/ {print $3}' < $TOP/revision.h`
 echo "Extracting agent/magent (with variable substitutions)"
 $spitshell >magent <<!GROK!THIS!
 $startperl
@@ -25,9 +24,9 @@ $startperl
 # via the filter. Mine looks like this:
 #   "|exec /users/ram/mail/filter >>/users/ram/.bak 2>&1"
 
-# $Id: magent.sh,v 3.0.1.17 2001/03/17 18:07:49 ram Exp ram $
+# $Id: magent.sh,v 3.0.1.14 1997/02/20 11:39:31 ram Exp $
 #
-#  Copyright (c) 1990-2006, Raphael Manfredi
+#  Copyright (c) 1990-1993, Raphael Manfredi
 #  
 #  You may redistribute only under the terms of the Artistic License,
 #  as specified in the README file that comes with the distribution.
@@ -36,17 +35,6 @@ $startperl
 #  of the source tree for mailagent 3.0.
 #
 # $Log: magent.sh,v $
-# Revision 3.0.1.17  2001/03/17 18:07:49  ram
-# patch72: mydomain and hiddennet now superseded by config vars
-# patch72: changed email_addr() and domain_addr() to honour new config vars
-#
-# Revision 3.0.1.16  1999/01/13  18:08:48  ram
-# patch64: changed agent_wait to AGENT_WAIT, now holding full path
-#
-# Revision 3.0.1.15  1997/09/15  15:05:06  ram
-# patch57: call new pmail() routine to process main message
-# patch57: fixed typo in -r usage
-#
 # Revision 3.0.1.14  1997/02/20  11:39:31  ram
 # patch55: used $* variable for no purpose
 #
@@ -130,7 +118,6 @@ $startperl
 # Current version number and patchlevel
 \$mversion = '$VERSION';
 \$patchlevel = '$PATCHLEVEL';
-\$revision = '$revision';
 
 # Want to lock mailboxes with flock ?
 \$lock_by_flock = '$lock_by_flock';
@@ -240,12 +227,9 @@ while ($ARGV[0] =~ /^-/) {
 		$log_level = int(shift);
 	}
 	elsif ($_ eq '-V') {	# Version number
-		print STDERR "$prog_name $mversion-$revision\n";
+		print STDERR "$prog_name $mversion PL$patchlevel\n";
 		exit 0;
 	}
-	elsif ($_ eq '-U') {	# Do not allow UNIQUE to reject / abort
-		++$disable_unique;
-	}
 	elsif ($_ eq '-TEST') {	# Mailagent run via TEST (undocumented feature)
 		++$test_mode;
 	}
@@ -269,12 +253,11 @@ $file_name = shift;				# File name to be processed (null if stdin)
 $ENV{'IFS'}='' if $ENV{'IFS'};	# Shell separation field
 &init_constants;				# Constants definitions
 &get_configuration;				# Get a suitable configuration package (cf)
-&patch_constants;				# Change some constants after config
 select(STDERR); $| = 1;			# In case we get perl warnings...
 select(STDOUT);					# and because the -t option writes on STDOUT,
 $| = 1;							# make sure it is flushed before we fork().
+$agent_wait = "agent.wait";		# Waiting file for out-of-the-queue mails
 $privlib = "$cf'home/../.." if $test_mode;	# Tests ran from test/out
-$AGENT_WAIT = "$cf'spool/agent.wait";		# Waiting file for mails
 
 $orgname = &tilda_expand($orgname);		# Perform run-time ~name substitution
 
@@ -360,53 +343,40 @@ if ($mbox_mail) {
 &read_stats;					# Load statistics into memory for fast update
 &newcmd'load if $cf'newcmd;		# Load user-defined command definitions
 
-#
-# If -q is not specfied, we need to process the file which was given to us
-# on the command line. We're calling pmail() to process it via locking,
-# but unfortunately we can't allow pmail() to unlink the processed file,
-# because it might be something the user wants to keep around...
-# However, if we were invoked by the filter program, the processed mail
-# will be unlinked later on. The trouble is the file was unlocked and
-# there is a slight time window were the message could be processed again by
-# another mailagent. If the 'queuehold' variable is reasonably set, such a
-# message will be skipped anyway, so it's not that critical.
-#
-
-my $process_queue = 1;
-
 if (!$run_queue) {				# Do not enter here if -q
-	if (0 != &pmail($file_name, 0)) {
-		&add_log("ERROR while processing main message--queing it") if $loglvl;
+	if (0 != &analyze_mail($file_name)) {	# Analyze the mail
+		&add_log("ERROR while processing main message--queing it")
+			if ($loglvl > 0);
 		&queue_mail($file_name, 'fm');
 		unlink $lockfile;
 		exit 0;					# Do not continue
-	} 
-
-	# If invoked from a tty and not in test mode, do not process queue
-	$process_queue = 0 if -t STDOUT && !$test_mode; 
+	} else {
+		$file = $file_name;		# Never corrupt $file_name
+		$file =~ s|.*/(.*)|$1|;	# Keep only basename
+		$file = "<stdin>" if $file eq '';
+		local($len) = 0 + $Header{'Length'};	# Force numeric value
+		&add_log("FILTERED [$file] $len bytes") if $loglvl > 4;
+	}
 }
 
-if ($process_queue) {
-	unless ($test_mode) {
-		# Fork a child: we have to take care of the filter script which is
-		# waiting for us to finish processing of the delivered mail.
-		&fork_child() unless $run_queue;
+unless ($test_mode) {
+	# Fork a child: we have to take care of the filter script which is waiting
+	# for us to finish processing of the delivered mail.
+	&fork_child() unless $run_queue;
 
-		# From now on, we are in the child process...
-		# Don't sleep at all if logging level is greater that 11
-		# or if $run_queue is true. Logging level of 12 and higher are
-		# for debugging and should not be used on a permanent basis
-		# anyway.
+	# From now on, we are in the child process... Don't sleep at all if logging
+	# level is greater that 11 or if $run_queue is true. Logging level of 12
+	# and higher are for debugging and should not be used on a permanent basis
+	# anyway.
 
-		$sleep = 1;					# Give others a chance to queue their mail
-		$sleep = 0 if $loglvl > 11 || $run_queue;
+	$sleep = 1;					# Give others a chance to queue their mail
+	$sleep = 0 if $loglvl > 11 || $run_queue;
 
-		do {						# Eventually process the queue
-			sleep 30 if $sleep;		# Wait in case new mail arrives
-		} while (&pqueue);
-	} else {
-		&pqueue;					# Process the queue once in test mode
-	}
+	do {						# Eventually process the queue
+		sleep 30 if $sleep;		# Wait in case new mail arrives
+	} while (&pqueue);
+} else {
+	&pqueue;					# Process the queue once in test mode
 }
 
 # Mailagent is exiting. Remove lock file as early as possible to avoid a
@@ -424,7 +394,7 @@ exit 0;
 # Print usage and exit
 sub usage {
 	print STDERR <<EOF;
-Usage: $prog_name [-dhilqtFIVU] [-s{umaryt}] [-f file] [-e rules] [-c config]
+Usage: $prog_name [-dhilqtFIV] [-s{umaryt}] [-f file] [-e rules] [-c config]
        [-L level] [-r file] [-o def] [mailfile]
   -c : specify alternate configuration file.
   -d : dump filter rules (special).
@@ -435,14 +405,13 @@ Usage: $prog_name [-dhilqtFIVU] [-s{umaryt}] [-f file] [-e rules] [-c config]
   -l : list message queue (special).
   -o : overwrite config file with supplied definition.
   -q : process the queue (special).
-  -r : specify alternate rule file.
+  -r : sepcify alternate rule file.
   -s : report gathered statistics (special).
   -t : track rules on stdout.
   -F : force processing on already filtered messages.
   -I : install configuration and perform sanity checks.
   -L : force logging level.
   -V : print version number and exits.
-  -U : prevent UNIQUE from rejecting an already processed Message-ID.
 EOF
 	exit 1;
 }
@@ -474,6 +443,7 @@ sub init_all {
 
 # Constants definitions
 sub init_constants {
+	require 'ctime.pl';
 	# Values for flock(), usually in <sys/file.h>
 	$LOCK_SH = 1;				# Request a shared lock on file
 	$LOCK_EX = 2;				# Request an exclusive lock
@@ -501,17 +471,15 @@ sub init_constants {
 	$BODY_INPUT = 1;			# Give body of mail as stdin
 	$MAIL_INPUT = 2;			# Pipe the whole mail
 	$HEADER_INPUT = 3;			# Pipe the header only
-	$MAIL_INPUT_BINARY = 4;		# Whole mail in binary (no transfer encoding)
 	$NO_FEEDBACK = 0;			# No feedback wanted
 	$FEEDBACK = 1;				# Feed result of command back into %Header
-	$FEEDBACK_ENCODING = 2;		# Same as $FEEDBACK, but probe body for encoding
 	
 	# The filter message
 	local($address) = &email_addr;
 	$FILTER =
-		"X-Filter: mailagent [version $mversion-$revision] for $address";
+		"X-Filter: mailagent [version $mversion PL$patchlevel] for $address";
 	$MAILER =
-		"X-Mailer: mailagent [version $mversion-$revision]";
+		"X-Mailer: mailagent [version $mversion PL$patchlevel]";
 
 	# For header fields alteration
 	$HD_STRIP = 0;				# Strip header fields
@@ -527,13 +495,6 @@ sub init_constants {
 	$MAX_LINKS = 100;			# Maximum number of symbolic link levels
 }
 
-# Change some constants after configuration file was parsed
-sub patch_constants {
-	local($address) = &email_addr;	# Will prefer cf vars to hardwired ones
-	$FILTER =
-		"X-Filter: mailagent [version $mversion-$revision] for $address";
-}
-
 # Initializes environment. All the variables are initialized in XENV array
 # The sole purpose of XENV is to be able to know what changes wrt the invoking
 # environment when dumping the rules. It also avoid modifying the environment
@@ -547,10 +508,9 @@ sub init_env {
 # List of special header keys which do not represent a true header field.
 sub init_pseudokey {
 	%Pseudokey = (
-		'Body', 1,			# Body of message
-		'Head', 1,			# Header of message
-		'All', 1,			# Concatenation of Header, "\n", Body
-		'=Body=', 1,		# Reference to body with decoded transfer encoding
+		'Body', 1,
+		'Head', 1,
+		'All', 1
 	);
 }
 
@@ -648,13 +608,7 @@ sub mbox_unlock {
 # Can't rely on the value of $cf'user since config file may not have
 # been parsed when this routine is first called. This routine is also used
 # to set a default value for $cf'email.
-# Once $cf'email exists however, its value is used.
 sub email_addr {
-	if (defined $cf'email) {
-		my $mail = $cf'email;
-		$mail .= '@' . &domain_addr unless $mail =~ /@/;
-		return $mail;
-	}
 	return $email_addr_cached if defined $email_addr_cached;
 	local($user);
 	($user) = getpwuid($>);
@@ -665,21 +619,12 @@ sub email_addr {
 }
 
 # Domain name address for current host
-# Use $cf'domain and $cf'hidenet when available.
 sub domain_addr {
 	local($_);							# Our host name
-	if (defined $cf'domain) {
-		$_ = $cf'domain;
-		if (lc($cf'hidenet) ne "on" || $_ eq '') {
-			$_ = &hostname;
-			$_ .= ".$cf::domain" unless /\./;
-		}
-	} else {
-		$_ = $hiddennet if $hiddennet ne '';
-		if ($_ eq '') {
-			$_ = &hostname;					# Must fork to get hostname, grr...
-			$_ .= $mydomain unless /\./;	# We want something fully qualified
-		}
+	$_ = $hiddennet if $hiddennet ne '';
+	if ($_ eq '') {
+		$_ = &hostname;					# Must fork to get hostname, grr...
+		$_ .= $mydomain unless /\./;	# We want something fully qualified
 	}
 	$_;
 }
@@ -702,10 +647,10 @@ sub mailbox_name {
 	# override value computed by Configure.
 	$maildir = $cf'maildrop if $cf'maildrop ne '';
 	# If Configure gave a valid 'maildir', use it. Otherwise compute one now.
-	unless ($maildir ne '' && -d $maildir) {
+	unless ($maildir ne '' && -d "$maildir") {
 		$maildir = "/usr/spool/mail";		# Default spooling area
-		-d $maildir || (-d "/usr/mail" && ($maildir = "/usr/mail"));
-		-d $maildir || ($maildir = $cf'home);
+		-d "/usr/mail" && ($maildir = "/usr/mail");
+		-d "$maildir" || ($maildir = "$cf'home");
 	}
 	local($mbox) = $cf'user;					# Default mailbox file name
 	$mbox = $cf'mailbox if $cf'mailbox ne '';	# Priority to config variable
@@ -755,8 +700,7 @@ sub fork_child {
 # Report any eval error and returns 1 if error detected.
 sub eval_error {
 	if ($@ ne '') {
-		$@ =~ s/ in file \(eval\) at line \d+//;	# Older perls
-		$@ =~ s/ at \(eval \d+\) line \d+\.//;		# Modern perl 5.x
+		$@ =~ s/ in file \(eval\) at line \d+//;
 		chop($@);
 		&add_log("ERROR $@") if $loglvl > 1;
 	}
@@ -764,7 +708,6 @@ sub eval_error {
 }
 
 !NO!SUBS!
-$grep -v '^;#' pl/ctime.pl >>magent
 $grep -v '^;#' pl/jobnum.pl >>magent
 $grep -v '^;#' pl/read_conf.pl >>magent
 $grep -v '^;#' pl/acs_rqst.pl >>magent
@@ -827,8 +770,5 @@ $grep -v '^;#' pl/biff.pl >>magent
 $grep -v '^;#' pl/rulenv.pl >>magent
 $grep -v '^;#' pl/options.pl >>magent
 $grep -v '^;#' pl/install.pl >>magent
-$grep -v '^;#' pl/base64.pl >>magent
-$grep -v '^;#' pl/qp.pl >>magent
-$grep -v '^;#' pl/termios/termios.pl >>magent
 chmod 755 magent
 $eunicefix magent
