@@ -1,4 +1,4 @@
-;# $Id: actions.pl,v 3.0.1.12 1995/09/15 14:01:17 ram Exp $
+;# $Id: actions.pl,v 3.0.1.15 1997/02/20 11:42:06 ram Exp $
 ;#
 ;#  Copyright (c) 1990-1993, Raphael Manfredi
 ;#  
@@ -9,6 +9,16 @@
 ;#  of the source tree for mailagent 3.0.
 ;#
 ;# $Log: actions.pl,v $
+;# Revision 3.0.1.15  1997/02/20  11:42:06  ram
+;# patch55: made 'perl -cw' clean and fixed a couple of typos
+;#
+;# Revision 3.0.1.14  1997/01/07  18:31:14  ram
+;# patch52: allow for @SH help to be understood, whatever the case
+;#
+;# Revision 3.0.1.13  1996/12/24  14:46:16  ram
+;# patch45: now reads 'help' as 'mailhelp' in command messages
+;# patch45: locate and perform security checks on launched executables
+;#
 ;# Revision 3.0.1.12  1995/09/15  14:01:17  ram
 ;# patch43: now escapes shell metacharacters for popen() on FORWARD and BOUNCE
 ;# patch43: will now make a note when delivering to an unlocked folder
@@ -276,6 +286,7 @@ sub process {
 			s/^mial(\w*)/mail$1/;		# Common mis-spellings
 			s/^mailpath/mailpatch/;
 			s/^mailist/maillist/;
+			s/^help/mailhelp/i;
 			# Now fetch command's name (first symbol)
 			if (/^([^ \t]+)[ \t]/) {
 				$first = $1;
@@ -592,7 +603,7 @@ sub send_message {
 	print MAILER "\n";			# Header separated from body
 	# Now write the body
 	local($tmp);				# Because of a bug in perl 4.0 PL19
-	while ($tmp = <MSG>) {
+	while (defined ($tmp = <MSG>)) {
 		next if $tmp =~ /^$/ && $. == 1;	# Escape sequence to protect header
 		&macros_subst(*tmp);		# In-place macro substitutions
 		print MAILER $tmp;			# Write message line
@@ -926,7 +937,7 @@ sub shell_command {
 	$SIG{'ALRM'} = 'DEFAULT';		# Restore default behaviour
 	local($msg) = $@;
 	$@ = '';						# Clear this global for our caller
-	if ($msg =~ /^failed/) {			# Something went wrong?
+	if ($msg =~ /^failed/) {		# Something went wrong?
 		&add_log("ERROR couldn't run '$program'") if $loglvl > 0;
 		return 1;					# Failed
 	} elsif ($msg =~ /^aborted/) {	# Writing to program failed
@@ -989,11 +1000,17 @@ sub alarm_clock {
 # Execute the command, ran in an eval to protect against SIGPIPE signals
 sub execute_command {
 	local($program, $input, $feedback) = @_;
+
+	local($location) = &locate_program($program);
+	die "can't locate $location in PATH\n" unless $location =~ m|/|;
+	die "unsecure $location\n" unless &exec_secure($location);
+
 	local($trace) = "$cf'tmpdir/trace.run$$";	# Where output goes
 	local($error) = "failed";				# Error reported by popen_failed
 	pipe(READ, WRITE);						# Open a pipe
 	local($pid) = fork;						# We fork here
 	$pid = -1 unless defined $pid;
+
 	if ($pid == 0) {						# Child process
 		alarm 0;
 		close WRITE;						# The child reads from pipe
@@ -1002,23 +1019,27 @@ sub execute_command {
 		unless (open(STDOUT, ">$trace")) {	# Where output goes
 			&add_log("WARNING couldn't create $trace: $!") if $loglvl > 5;
 			if ($feedback == $FEEDBACK) {	# Need trace if feedback
-				kill 'SIGPIPE', $ppid;		# Parent still waiting
+				kill 'SIGPIPE', getppid;	# Parent still waiting
 				exit 1;
 			}
 		}
 		open(STDERR, ">&STDOUT");			# Make it follow pipe
-		exec $program;						# Run the program now
+		# Using a sub-block ensures exec() is followed by nothing
+		# and makes mailagent "perl -cw" clean, whatever that means ;-)
+		{ exec $program }					# Run the program now
 		&add_log("ERROR couldn't exec '$program': $!") if $loglvl > 1;
 		exit 1;
 	} elsif ($pid == -1) {
 		&add_log("ERROR couldn't fork: $!") if $loglvl;
 		return;
 	}
+
 	close READ;								# The parent writes to its child
 	$error = "aborted";						# Error reported by popen_failed
 	select(WRITE);
 	$| = 1;									# Hot pipe wanted
 	select(STDOUT);
+
 	# Now feed the program with the mail
 	if ($input == $BODY_INPUT) {			# Pipes body
 		print WRITE $Header{'Body'};
@@ -1028,6 +1049,7 @@ sub execute_command {
 		print WRITE $Header{'Head'};
 	}
 	close WRITE;							# Close input, before waiting!
+
 	wait();									# Wait for our child
 	local($status) = $? ? "failed" : "ok";
 	if ($?) {
@@ -1040,6 +1062,7 @@ sub execute_command {
 			die "feedback\n";				# Longjmp to shell_command
 		}
 	}
+
 	&handle_output;			# Take appropriate action with command output
 	unlink "$trace";		# Remove output of command
 	die "non-zero status\n" unless $status eq 'ok';
@@ -1451,7 +1474,6 @@ sub after {
 	local($time, $action) = @_;
 	local($no_input) = $opt'sw_n;
 	local($shell_cmd) = $opt'sw_s;
-	local($cmd_cmd) = $opt'sw_c;
 	local($agent_cmd) = $opt'sw_a || !($opt'sw_n || $opt'sw_s || $opt'sw_c);
 	local($now) = time;					# Current time
 	local($start);						# Action's starting time
