@@ -1,6 +1,6 @@
-;# $Id: rulenv.pl 69 2011-05-18 12:28:03Z rmanfredi $
+;# $Id: rulenv.pl,v 3.0.1.3 1995/08/07 16:24:06 ram Exp $
 ;#
-;#  Copyright (c) 1990-2006, Raphael Manfredi
+;#  Copyright (c) 1990-1993, Raphael Manfredi
 ;#  
 ;#  You may redistribute only under the terms of the Artistic License,
 ;#  as specified in the README file that comes with the distribution.
@@ -61,7 +61,7 @@ sub init {
 # Set-up initial environment for rules.
 # This routine is called once for every mail parsed.
 sub setup {
-	&init unless %Spec;
+	&init unless defined %Spec;
 	eval $SETUP if $SETUP ne '';
 	&'add_log("ERROR env'setup: $@") if $@;
 	undef %Var;
@@ -106,7 +106,7 @@ sub undef {
 # If an action is required by the resetting of a variable, it is performed
 # following the directive from the %Spec table.
 sub restore {
-	return unless %Var;
+	return unless defined %Var;
 	local($code) = '';		# Code built to restore original variable values
 	foreach $var (keys %Var) {
 		$code .= "\$$var = \$Var{'$var'};\n";
