@@ -1,6 +1,6 @@
-# $Id: patbounce.pl 1 2006-08-24 13:24:12Z rmanfredi $
+# $Id: patbounce.pl,v 3.0.1.2 1997/01/07 18:37:43 ram Exp $
 #
-#  Copyright (c) 1990-2006, Raphael Manfredi
+#  Copyright (c) 1990-1993, Raphael Manfredi
 #  
 #  You may redistribute only under the terms of the Artistic License,
 #  as specified in the README file that comes with the distribution.
@@ -9,9 +9,6 @@
 #  of the source tree for mailagent 3.0.
 #
 # $Log: patbounce.pl,v $
-# Revision 3.0.1.3  1997/09/15  15:20:33  ram
-# patch57: also handle bounced output from @SH package
-#
 # Revision 3.0.1.2  1997/01/07  18:37:43  ram
 # patch52: strip out trailing > on addresses, since matching is greedy
 #
@@ -179,12 +176,6 @@ sub pbounce_failaddr {
 # Subject: Patch 42 for mailagent version 3.0 has been released.
 # Subject: mailagent 3.0 patch #45
 #
-# If the above fail, then it may be an "@SH package" command that failed.
-# They requested 'mailpatches' or 'notifypatches' and our reply to them
-# failed miserably. Look for:
-#
-#	package xxx@yyy mailagent 3.0 ...
-#
 # Returns ($package, $version) or an empty array if we can't figure it out.
 #
 sub pbounce_package {
@@ -193,8 +184,6 @@ sub pbounce_package {
 		/^Subject:\s+Patch.*for\s+([\w-]+)\s+version\s+([\d.]+)/;
 	return ($1, $2) if $header{'Body'} =~
 		/^Subject:\s+([\w-]+)\s+([\d.]+)\s+patch #\d+/;
-	return ($1, $2) if $header{'Body'} =~
-		/^\s+package\s+\S+\s+([\w-]+)\s+([\d.]+)/;
 	return ();
 }
 
